/* This is "heat2.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and test driver for the
   heat transfer problem described in Chapter 17 of the text. */


#include <stdio.h>



/* This structure is designed to keep track of the seven segment
   boundary temperatures in a six-segment finite-element simulation. */

typedef struct {
  double seg0;
  double seg1;
  double seg2;
  double seg3;
  double seg4;
  double seg5;
  double seg6;
} segments;



/* This function models heat transfer in a rod of uniform temperature
   that is insulated along its lengths and has a heat source applied
   to its ends.  The rod's thermal "diffusivity" in cm^2/sec and
   "length" in cm are given, as are the temperature of the "rod", the
   "left" heat source, and the "right" heat source in degrees Celsius.

   The function returns the approximate temperature of the center of
   the rod in degrees Celsius "duration" seconds after the heat sources
   are applied.

   The implementation is based on a finite-element model with six
   elements. */

double simulate (double diffusivity, double length,
                 double left, double right, double rod,
                 double duration)
{
  double delta;   /* Length in seconds of each simulated time step */

  segments cur;   /* Temperatures at the ends of each of the six */
                  /* segments, where cur.seg0 is the temperature of */
                  /* the left heat source and cur.seg6 is the */
                  /* temperature of the right heat source. */

  segments nxt;   /* Used to temporarily hold the new temperatures */
                  /* of cur.seg1 through cur.seg5 during the simulation. */


  /* Compute the length of the simulated time step. */

  delta = (length*length/36) / (2*diffusivity);


  /* Initialize "cur" to reflect the temperatures at time zero. */

  cur.seg0 = left;
  cur.seg1 = rod;
  cur.seg2 = rod;
  cur.seg3 = rod;
  cur.seg4 = rod;
  cur.seg5 = rod;
  cur.seg6 = right;


  /* Simulate the passage of time.  During each iteration, compute the
     new segment boundary temperatures in terms of the old segment
     boundary temperatures. */

  while (duration > 0) {

    nxt.seg1 = (cur.seg0 + cur.seg2) / 2;
    nxt.seg2 = (cur.seg1 + cur.seg3) / 2;
    nxt.seg3 = (cur.seg2 + cur.seg4) / 2;
    nxt.seg4 = (cur.seg3 + cur.seg5) / 2;
    nxt.seg5 = (cur.seg4 + cur.seg6) / 2;

    cur.seg1 = nxt.seg1;
    cur.seg2 = nxt.seg2;
    cur.seg3 = nxt.seg3;
    cur.seg4 = nxt.seg4;
    cur.seg5 = nxt.seg5;

    duration = duration - delta;
  }


  /* Return the temperature of the center segment boundary. */

  return(cur.seg3);

}



/* Test driver for "simulate". */

void main (void)
{

  double C, length, left,   /* Constants specified by the user. */
         right, rod, time,
         temp;


  /* Read constants from the keyboard using "scanf". */

  printf("Enter thermal diffusivity (cm^2/sec): ");
  scanf("%lf", &C);

  printf("Enter length of rod (cm): ");
  scanf("%lf", &length);

  printf("Enter temperature at left end (degrees C): ");
  scanf("%lf", &left);

  printf("Enter temperature of rod (degrees C): ");
  scanf("%lf", &rod);

  printf("Enter temperature at right end (degrees C): ");
  scanf("%lf", &right);

  printf("Enter length of simulation (sec): ");
  scanf("%lf", &time);


  /* Calculate and display the result. */

  temp = simulate(C, length, left, right, rod, time);
  printf("Temperature at center after %g seconds is %g degrees C\n",
         time, temp);

}
